// src/backend/dynamic_process.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef DYNAMIC_PROCESS_HH
#define DYNAMIC_PROCESS_HH 1

#include "config.h"
#include "gettext.h"

#include "process.hh"
#include "dynamic_schedulable.hh"
#include "static_process.hh"

#include "smartp.tcc"

#include <glibmm/ustring.h>
#include <vector>

namespace sgpem
{
  class DynamicProcess;
  class StaticProcess;
  class DynamicThread;
  class Thread;

  class SG_DLLLOCAL DynamicProcess : public DynamicSchedulable, public Process
  {
  public:
    DynamicProcess(StaticProcess* core);
    DynamicProcess(const DynamicProcess &other);
		virtual ~DynamicProcess();

    std::vector<Thread*> get_threads();
    std::vector<const Thread*> get_threads() const;

    state get_state() const;

    void serialize(SerializeVisitor& translator) const;

    int get_last_acquisition() const;
    int get_last_release() const;
    unsigned int get_elapsed_time() const;

    virtual StaticProcess& get_core();
    virtual const StaticProcess& get_core() const;

		// Does also the job of "add_thread" and "remove_thread"
		std::vector<DynamicThread*>& get_dynamic_threads();

  private:
    memory::smart_ptr<StaticProcess> _core;
    std::vector<DynamicThread*> _dynamic_threads;
  };

}

#endif
