// src/backend/dynamic_request.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef DYNAMIC_REQUEST_HH
#define DYNAMIC_REQUEST_HH 1

#include "config.h"

#include "request.hh"
#include "static_request.hh"

#include "smartp.hh"

#include <vector>

namespace sgpem
{
  class DynamicRequest;
  class SerializeVisitor;
  class DynamicThread;
  class SubRequest;
  class DynamicSubRequest;

  class SG_DLLLOCAL DynamicRequest : public Request
  {
  public:
    DynamicRequest(StaticRequest *core, DynamicThread* owner);
    DynamicRequest(const DynamicRequest& other, DynamicThread* owner);
		~DynamicRequest();

		virtual bool operator==(const Request& op2) const;

		virtual std::vector<SubRequest*>  get_subrequests();

    DynamicThread& get_thread();

    unsigned int get_instant() const;

    state get_state() const;

    void serialize(SerializeVisitor& translator) const;

		StaticRequest& get_core();
		const StaticRequest& get_core() const;

		// Since this method is visible only by the backend,
		// return directly a reference that lets us to
		// add and remove subrequests at will.
    std::vector<DynamicSubRequest*>& get_dynamic_subrequests();

  private:
    // Undefined
    DynamicRequest(const DynamicRequest& other);

    memory::smart_ptr<StaticRequest> _static_request;
    DynamicThread* _dynamic_thread;
    std::vector<DynamicSubRequest*> _dynamic_subrequests;
  };

}

#endif

