// src/global_preferences.cc - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "config.h"
#include "key_file.hh"
#include "global_preferences_serializer.hh"
#include <sstream>

namespace sgpem
{

  GlobalPreferencesSerializer::GlobalPreferencesSerializer(GlobalPreferences &gp)
    : _globalPreferences(gp)
  {}
  
  
  
  void
  GlobalPreferencesSerializer::file_read(const Glib::ustring& filename)
  {
    _globalPreferences._mod_dirs.clear();
    _globalPreferences._pol_dirs.clear();
    KeyFile::file_read(filename);
    // read modules directories
    {
      const Glib::ustring* val = search_value(Glib::ustring("modules-dir-number"));
      if(val)
      {
        std::istringstream istr(val->c_str());
        int n;
        istr >> n;
        for(int i=1; i<=n; i++)
        {
          std::ostringstream ostr;
          ostr << "modules-dir-" << i << std::ends;
          Glib::ustring key(ostr.str());
          val = search_value(key);
          if(val)
          {
            _globalPreferences.add_modules_dir(*val);
          }
        }
      }
    }

    // read policies directories
    {
      const Glib::ustring* val = search_value(Glib::ustring("policies-dir-number"));
      if(val)
      {
        std::istringstream istr(val->c_str());
        int n;
        istr >> n;
        for(int i=1; i<=n; i++)
        {
          std::ostringstream ostr;
          ostr << "policies-dir-" << i << std::ends;
          Glib::ustring key(ostr.str());
          val = search_value(key);
          if(val)
          {
            _globalPreferences.add_policies_dir(*val);
          }
        }
      }
    }
  }
  
  void
  GlobalPreferencesSerializer::file_write(const Glib::ustring& filename)
  {
    // add modules directories
    {
      GlobalPreferences::dir_iterator iter=_globalPreferences.modules_dir_begin();
      int n=0;
      while(iter!=_globalPreferences.modules_dir_end()){
        std::ostringstream ostr;
        n++;
        ostr << "modules-dir-" << n << std::ends;
        Glib::ustring key(ostr.str());
        insert_key_value(key, (*iter));
        ++iter;
      }
      Glib::ustring key("modules-dir-number");
      std::ostringstream ostr;
      ostr << n << std::ends;
      Glib::ustring value(ostr.str());
      insert_key_value(key, value);
    }
    // add policies directories
    {
      GlobalPreferences::dir_iterator iter=_globalPreferences.policies_dir_begin();
      int n=0;
      while(iter!=_globalPreferences.policies_dir_end()){
        std::ostringstream ostr;
        n++;
        ostr << "policies-dir-" << n << std::ends;
        Glib::ustring key(ostr.str());
        insert_key_value(key, (*iter));
        ++iter;
      }
      Glib::ustring key("policies-dir-number");
      std::ostringstream ostr;
      ostr << n << std::ends;
      Glib::ustring value(ostr.str());
      insert_key_value(key, value);
    }
    
    KeyFile::file_write(filename);
  }
}
