// src/backend/key_file.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef GLOBAL_PREFERENCES_SERIALIZER_HH
#define GLOBAL_PREFERENCES_SERIALIZER_HH 1

#include "global_preferences.hh"
#include "key_file.hh"

namespace sgpem
{
  class GlobalPreferencesSerializer;


  
  /** \brief Save and retrieve global preferences formatted as key=value rows.
   *
   * This class handles files to store and retrieve global preferences
   * in the form of: key=value rows.
   *
   */
  class SG_DLLEXPORT GlobalPreferencesSerializer : public KeyFile
  {

  public:
    /** \brief Object constructor */
    GlobalPreferencesSerializer(GlobalPreferences &gp);

    /** \brief Read a file into this object.
     *
     * \param filename The file to read from.
     */
    void file_read(const Glib::ustring& filename);

    /** \brief Write a file from this object.
     *
     * \param filename The file to write to.
     */
    void file_write(const Glib::ustring& filename);
  private:
    GlobalPreferences& _globalPreferences;
  };
}

#endif
