// src/backend/policies_gatekeeper.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef POLICIES_GATEKEEPER_HH
#define POLICIES_GATEKEEPER_HH 1

namespace sgpem
{
  class PolicyManager;
  class Policy;
  class History;
}

#include "config.h"

#include <vector>
#include <map>
#include <stdexcept>

#include "singleton.hh"

namespace sgpem
{
  class PoliciesGatekeeper;

  /** \brief FIXME document me


   */

  class SG_DLLEXPORT PoliciesGatekeeper : public Singleton<PoliciesGatekeeper>
  {
    friend class Singleton<PoliciesGatekeeper>;

  public:
    std::vector<PolicyManager*>     get_registered() const;

    void       register_manager(PolicyManager* manager);

    void       unregister_manager(PolicyManager* manager);

    Policy*       get_current_policy(History* history) throw(std::runtime_error);

    void       activate_policy(History* history, Policy* policy);

  private:
    PoliciesGatekeeper();     //private constructor.
    PoliciesGatekeeper(const PoliciesGatekeeper&);
    PoliciesGatekeeper& operator=(const PoliciesGatekeeper&);

    // Deactivates active policies managed by the specified manager.
    void       deactivate_policies(PolicyManager* manager);

    std::vector<PolicyManager*>     _registered;
    std::map<History*, Policy*>     _active_policies;
  };

}//~ namespace sgpem

#endif //POLICIES_GATEKEEPER_HH

