// src/graphicalterminalio.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef GRAPHICALTERMINALIO_HH
#define GRAPHICALTERMINALIO_HH 1

#include "config.h"
#include "gettext.h"

#include <gtkmm/box.h>
#include <gtkmm/entry.h>
#include <gtkmm/textview.h>
#include <gtkmm/window.h>
#include <glibmm/thread.h>
#include <glibmm/ustring.h>

#include "io_manager.hh"
#include "text_simulation.hh"

#include <memory>

namespace sgpem
{

  // ---------------------------------------------

  class GraphicalTerminalIO;

  // ---------------------------------------------

  /** \brief Class GraphicalTerminalIO implements the methods requested
   by IOManager using graphichal widgets.
   Consists in a window displaying some textual output and providing a control for text input.
   It is associated with a TextSimulaton, which provide the interpretation and elaboration of the user
   commands.
    */
  class GraphicalTerminalIO : public IOManager, public Gtk::Window
  {

  public:

    /**
    Creates a GraphichalTerminalIO supported by the specified TextSimulation.
    \param sim the TextSimulation that will support this IOManager.
    */
    GraphicalTerminalIO(TextSimulation* sim);

    /**
    Destructor.
    */
    virtual ~GraphicalTerminalIO();

    /**
    Prints on the window the provided buffer.
    \param buffer the buffer to print.
    */
    virtual uint   write_buffer(const Glib::ustring& buffer);

    /**
        Reads the user input.
    \return the user input.
    */
    virtual Glib::ustring  read_command();

    /**
    Returns whether the IOManager can read user input while performing a write operation.
    \return whether the IOManager can read user input while performing a write operation.
    */
    virtual bool   is_full_duplex();

    /**
    Specifies the operations to perform at user input.
    */
    void      onSend();

  private:
    TextSimulation*       _sim;
    Gtk::TextView         _text_output;
    mutable Gtk::Entry    _text_input;
    Glib::Mutex           _mtx;
  };

}

#endif

