// src/iomanager.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef GTKGUI_IOMANAGER_HH
#define GTKGUI_IOMANAGER_HH 1

#include "config.h"
#include "gettext.h"

#include <glibmm/ustring.h>

namespace sgpem
{

  class IOManager;

  /** \brief Class IOManager is an interface for controlling textual I/O.

  */
  class IOManager
  {
  public:
    virtual ~IOManager() {}

    /**Writes a string into an output (the console, a text widget, ...)
       \returns the number of written characters
    */
    virtual uint   write_buffer(const Glib::ustring& buffer) = 0;

    /**Reads a command from an interactive input (the console, a text widget, ...)
       \returns a trimmed string (without blank spaces, tabs... at the extremities) 
    */
    virtual Glib::ustring read_command() = 0;

    /**Specify whether this IOManger permits to write and read at the same time
     */
    virtual bool   is_full_duplex() = 0;
  };

}

#endif
