// src/standard_io.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef STANDARD_IO_HH
#define STANDARD_IO_HH 1

#include "config.h"
#include "gettext.h"

#include <iostream>
#include <glibmm/ustring.h>

#include "io_manager.hh"

namespace sgpem
{


  class StandardIO;

  /**
    Concrete subclass of \ref IOManager implemented using
    standard input and standard output.
  */
  class StandardIO : public IOManager
  {
  public:

    /**
      Prints the parameter to standard output.
      \param buffer The string to print.
      \return The number of printed characters.
     */
    uint  write_buffer(const Glib::ustring& buffer);

    /**
      Reads from standard input a command.
      Whitespaces are trimmed at the beginning and at the end.
      \return The command string, trimmed of whitespaces.
     */
    Glib::ustring read_command();

    /**
      Returns whether this device can simultaneously read and write.
      \return true if the device can read and write simultaneously, false otherwise.
     */
    bool  is_full_duplex();
  };

}

#endif
