// src/testsuite/test-stepforward.cc - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


#include "history.hh"

using namespace sgpem;


        memory::smart_ptr<sgpem::DynamicSchedulable>
                History::get_scheduled_at(int time) const
        {
            using namespace memory;
            smart_ptr<DynamicSchedulable> scheduled_at = smart_ptr<DynamicSchedulable>();
            if (0 <= time && time <= _total_time_elapsed)
            {
                smart_ptr<ReadyQueue> sl = get_simulation_status_at(time);
                bool found = false;
                bool invalid = sl == smart_ptr<ReadyQueue>();
                for (uint i = 0; !found && !invalid && i < sl->size(); i++)
                {
                    const DynamicSchedulable* ss = sl->get_item_at(i);
                    if ((bool)ss && ss->get_state() == DynamicSchedulable::state_running)
                    {
                        scheduled_at = smart_ptr<DynamicSchedulable>(new DynamicSchedulable(*(ss)));
                        found = true;
                    }
                }
            }
            return scheduled_at;
        }


        memory::smart_ptr<sgpem::ReadyQueue>
                History::get_simulation_status_at(int time) const
        {
            using namespace memory;
            smart_ptr<ReadyQueue> simulation_status_at = smart_ptr<ReadyQueue>();
            if (0 <= time && time <= _total_time_elapsed)
            {
                if (_slice == memory::smart_ptr<Slice>())
                    std::cout<<"History::get_simulation_status_at.NULL.error";
                else
                    simulation_status_at = memory::smart_ptr<ReadyQueue>
                    (
                        new ReadyQueue
                        (
                            *(_slice->get_simulation_status())
                        )
                    );
            }
            return simulation_status_at;
        }


        int
                History::get_current_time() const
        {
            return _total_time_elapsed;
        }


        void
                History::enqueue_slice(const sgpem::ReadyQueue& status)
        {
            _slice = memory::smart_ptr<Slice>(new Slice(_total_time_elapsed, 1, status));
            _total_time_elapsed++;
        }


        void
                History::truncate_at(int instant)
        {
            //std::cout << "\nRecreating a Singleton History";
            _slice = memory::smart_ptr<Slice>();
            _total_time_elapsed = -1;
        }


        History&
        History::get_instance()
        {
            if (History::_instance == NULL)
                History::_instance = new History();
            return *History::_instance;
        }


        History::History()
        {
            _slice = memory::smart_ptr<Slice>();
            _total_time_elapsed = -1;
        }


    History * History::_instance = NULL;
