// src/testsuite/test-stepforward.cc - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "prrpolicy.hh"

using namespace sgpem;

PRRPolicy::PRRPolicy()
{
    _instance = this;
}

PRRPolicy::PRRPolicy(int quantum)
    : _quantum(quantum)
{
    _instance = this;
}

Policy&
PRRPolicy::get_instance()
{
    if(_instance == NULL) _instance = new PRRPolicy(3); // quantum size
    return *_instance;
}


PRRPolicy::~PRRPolicy()
{}

  void
PRRPolicy::configure()
        throw(UserInterruptException)
{}

  void
PRRPolicy::sort_queue() const
    throw(UserInterruptException)
{
    ReadyQueue* local_sl = Scheduler::get_instance().get_ready_queue();
    for (uint useless = 0; useless < local_sl->size(); useless++)
        for (uint i = 0; i < local_sl->size() - 1; i++)
                if
                (
                    local_sl->get_item_at(i)->get_schedulable()->get_priority() >
                    local_sl->get_item_at(i + 1)->get_schedulable()->get_priority()
                )
                    local_sl->swap(i, i + 1);
}

  void 
PRRPolicy::activate()
{}

  void 
PRRPolicy::deactivate()
{}

  int
PRRPolicy::get_id() const
{
    return 42;
}

  sgpem::policy_sorts_type
PRRPolicy::wants() const
    throw(UserInterruptException)
{
    return policy_sorts_processes;
}

  Glib::ustring
PRRPolicy::get_name() const
{
    return "42";
}

  Glib::ustring
PRRPolicy::get_description() const
{
    return "42";
}

  bool
PRRPolicy::is_pre_emptive() const
        throw(UserInterruptException)
{
    return 1;
}

  int
PRRPolicy::get_time_slice() const
        throw(UserInterruptException)
{
    return _quantum;
}

  PolicyParameters&
PRRPolicy::get_parameters()
{
    return _parameters;
}

Policy*
PRRPolicy::_instance = NULL;
